@@ -2,6 +2,7 @@ import typer
 from pathlib import Path
 import xml.etree.ElementTree as ET
 from xml.dom import minidom
+import json
 
 app = typer.Typer()
 
@@ -10,8 +11,10 @@ def make_dynamic(element, key):
     _id = element.get("id")
     if key is None:
         value = element.text
-        s = f"{_id}={value}"
-        element.text = f"$({s})"
+        if value:
+            s = f"{_id}={value}"
+            element.text = f"$({s})"
+            return (_id, value)
     else:
         value = element.get(key)
         if "$" in value:
@@ -23,15 +26,20 @@ def make_dynamic(element, key):
             # the way that defaults are passed means you can't use '=' in there values...
             # https://github.com/CompEvol/beast2/pull/991
             return None
-        s = f"{_id}.{key}={value}"
+        _id = f"{_id}.{key}"
+        s = f"{_id}={value}"
         element.set(key, f"$({s})")
+        return (_id, value)
 
 
 def make_all_dynamic(element):
-    for key in filter(lambda k: k != "id", element.keys()):
+    json_vars = [
         make_dynamic(element, key)
+        for key in filter(lambda k: k != "id", element.keys())
+    ]
     if element.tag == "parameter":
-        make_dynamic(element, None)
+        json_vars.append(make_dynamic(element, None))
+    return json_vars
 
 
 def add_mc3_options(run):
@@ -113,23 +121,9 @@ def apply_optimise(path_to_output: Path, run):
         op_el.set(key, f"$({s})")
 
 
-@app.command()
-def main(
-    beast_xml: Path,
-    outfile: Path = typer.Option(None, help="Path to save the dynamic BEAST XML file."),
-    mc3: bool = typer.Option(False, help="Add default MC3 options to XML file."),
-    ps: bool = typer.Option(False, help="Add default PathSampler options to XML file."),
-    ns: bool = typer.Option(
-        False, help="Add default Multi threaded nested sampling options to XML file."
-    ),
-    optimise: Path = typer.Option(
-        None,
-        help="Path to previous run output file. Operator optimisations will be extracted.",
-    ),
+def create_dynamic_xml(
+    beast_xml, outfile=None, mc3=False, ps=False, ns=False, optimise=None, json_out=None
 ):
-    """
-    Dynamic BEAST XML
-    """
     tree = ET.parse(beast_xml)
     root = tree.getroot()
     run = root.find("run")
@@ -144,10 +138,11 @@ def main(
     if ns:
         add_ns_options(run)
 
+    json_vars = []
     for el in run.iter():
         if "idref" in el.keys():
             continue
-        make_all_dynamic(el)
+        json_vars += make_all_dynamic(el)
 
     if optimise:
         apply_optimise(optimise, run)
@@ -157,3 +152,31 @@ def main(
     else:
         xmlstr = minidom.parseString(ET.tostring(root)).toprettyxml(newl="")
         typer.echo(xmlstr)
+    if json_out:
+        json_vars = [v for v in json_vars if v]
+        with open(json_out, "w") as f:
+            json.dump(dict(json_vars), f, ensure_ascii=False, indent=4)
+
+
+@app.command()
+def main(
+    beast_xml: Path,
+    outfile: Path = typer.Option(None, help="Path to save the dynamic BEAST XML file."),
+    mc3: bool = typer.Option(False, help="Add default MC3 options to XML file."),
+    ps: bool = typer.Option(False, help="Add default PathSampler options to XML file."),
+    ns: bool = typer.Option(
+        False, help="Add default Multi threaded nested sampling options to XML file."
+    ),
+    optimise: Path = typer.Option(
+        None,
+        help="Path to previous run output file. Operator optimisations will be extracted.",
+    ),
+    json_out: Path = typer.Option(
+        None,
+        help="Create a json file of all dynamic variables. For use with the -DF beast argument.",
+    ),
+):
+    """
+    Dynamic BEAST XML
+    """
+    create_dynamic_xml(beast_xml, outfile, mc3, ps, ns, optimise, json_out)
